;********************************************************
;
;               Week LED control for RCC
;
;                                 Device : PIC16F628
;                                 Clock  : 4MHz(INT)
;                                 Author : Seiichi Inoue
;********************************************************

        list            p=pic16f628
        include         p16f628.inc
        __config        h'3F10' ;OSC is Int 4MHz
                                ;RA5,RB4,RB6,RB7 are I/O
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Brown-out detection OFF
                                ;Watchdog timer OFF
        errorlevel      -302    ;Eliminate bank warning

;****************  Label Definition  ********************
        cblock  h'20'
disp1                           ;1st line data
disp2                           ;2nd line data
disp3                           ;3rd line data
disp4                           ;4th line data
disp5                           ;5th line data
disp6                           ;6th line data
disp7                           ;7th line data
disp8                           ;8th line data
line_indexr                     ;Index for display
line_indexw                     ;Index for set
input_data                      ;Input data save area
line_data                       ;Data save area
        endc

disp    equ     disp1           ;Display table head adr

;***************** Program Start ************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;******************** Dispaly data **********************

sunday                          ;Index = 0
        retlw   b'11111111'     ;1st line data
        retlw   b'00000001'     ;2nd line data
        retlw   b'01101101'     ;3rd line data
        retlw   b'01101101'     ;4th line data
        retlw   b'01101101'     ;5th line data
        retlw   b'01101101'     ;6th line data
        retlw   b'00000001'     ;7th line data
        retlw   b'11111111'     ;8th line data

monday                          ;Index = 1
        retlw   b'11111111'     ;1st line data
        retlw   b'00000000'     ;2nd line data
        retlw   b'01010111'     ;3rd line data
        retlw   b'01010111'     ;4th line data
        retlw   b'01010111'     ;5th line data
        retlw   b'00000001'     ;6th line data
        retlw   b'11111110'     ;7th line data
        retlw   b'11111111'     ;8th line data

tuesday                         ;Index = 2
        retlw   b'10111110'     ;1st line data
        retlw   b'11011101'     ;2nd line data
        retlw   b'11111011'     ;3rd line data
        retlw   b'00000111'     ;4th line data
        retlw   b'11111011'     ;5th line data
        retlw   b'11011101'     ;6th line data
        retlw   b'10111110'     ;7th line data
        retlw   b'11111111'     ;8th line data

wednesday                       ;Index = 3
        retlw   b'11011011'     ;1st line data
        retlw   b'11000111'     ;2nd line data
        retlw   b'11111111'     ;3rd line data
        retlw   b'00000000'     ;4th line data
        retlw   b'11111110'     ;5th line data
        retlw   b'11001111'     ;6th line data
        retlw   b'11010111'     ;7th line data
        retlw   b'11111011'     ;8th line data

thursday                        ;Index = 4
        retlw   b'11011101'     ;1st line data
        retlw   b'11011011'     ;2nd line data
        retlw   b'11010111'     ;3rd line data
        retlw   b'00000000'     ;4th line data
        retlw   b'11010111'     ;5th line data
        retlw   b'11011011'     ;6th line data
        retlw   b'11011101'     ;7th line data
        retlw   b'11111111'     ;8th line data

friday                          ;Index = 5
        retlw   b'11101110'     ;1st line data
        retlw   b'11010100'     ;2nd line data
        retlw   b'10010110'     ;3rd line data
        retlw   b'01000000'     ;4th line data
        retlw   b'10010110'     ;5th line data
        retlw   b'11010100'     ;6th line data
        retlw   b'11101110'     ;7th line data
        retlw   b'11111111'     ;8th line data

saturday                        ;Index = 6
        retlw   b'11111101'     ;1st line data
        retlw   b'11011101'     ;2nd line data
        retlw   b'11011101'     ;3rd line data
        retlw   b'00000001'     ;4th line data
        retlw   b'11011101'     ;5th line data
        retlw   b'11011101'     ;6th line data
        retlw   b'11111101'     ;7th line data
        retlw   b'11111111'     ;8th line data

light_on                        ;Index = 7
        retlw   b'00000000'     ;1st line data
        retlw   b'00000000'     ;2nd line data
        retlw   b'00000000'     ;3rd line data
        retlw   b'00000000'     ;4th line data
        retlw   b'00000000'     ;5th line data
        retlw   b'00000000'     ;6th line data
        retlw   b'00000000'     ;7th line data
        retlw   b'00000000'     ;8th line data

;******************* Initial Process ********************
init
;*** Set Port mode
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00111000'     ;RA2-0:OUT RA3-5:IN mode
        movwf   trisa           ;Set TRISA reg
        movlw   b'00000000'     ;RB7-0:OUT mode
        movwf   trisb           ;Set TRISB reg

;*** Set Option reg
        movlw   b'00000100'     ;PS=1:32
        movwf   option_reg      ;Set OPTION_REG

;*** Set PIE1 reg
        movlw   b'00000000'     ;TMR1IE=OFF
        movwf   pie1            ;Set PIE1 reg
        bcf     status,rp0      ;Change to Bank0

;*** Set CMCON reg
        movlw   b'00000111'     ;RA port to digital
        movwf   cmcon           ;Set CMCON reg

;*** Set TMR0 reg
        movlw   d'225'          ;256-1000us/32us=225
        movwf   tmr0            ;Set TMR0

;*** Other registers and work area initialization
        clrf    porta           ;Clear PORTA
        clrf    portb           ;Clear PORTB
        clrf    line_indexr     ;Clear index for dip
        clrf    line_indexw     ;Clear index for set
        movlw   b'11111111'     ;Set light off data
        movwf   disp1           ;Set 1st line
        movwf   disp2           ;Set 2nd line
        movwf   disp3           ;Set 3rd line
        movwf   disp4           ;Set 4th line
        movwf   disp5           ;Set 5th line
        movwf   disp6           ;Set 6th line
        movwf   disp7           ;Set 7th line
        movwf   disp8           ;Set 8th line

;*** Set interrupt condition
        movlw   b'10100000'     ;GIE,TOIE=1
        movwf   intcon          ;Interruption enable

        goto    $               ;Wait interruption

int
;***** TMR0 Interruption ( 1 millisecond interval) ******
        bcf     intcon,t0if     ;Clear timer int flag
        movlw   d'225'          ;Set 1 millisecond
        movwf   tmr0            ;Set TMR0

        movfw   line_indexr     ;Read index
        movwf   porta           ;Set display index
        
        movlw   disp            ;Read disp table head adr
        addwf   line_indexr,w   ;RA table head + index
        movwf   fsr             ;Set table address
        movfw   indf            ;Read display data
        movwf   portb           ;Set data to PORTB
        incf    line_indexr,f   ;Index + 1
        movfw   line_indexr     ;Read index
        sublw   d'7'            ;Check upper limit
        btfsc   status,c        ;Over ?
        goto    int_end         ;No. Jump to end
        clrf    line_indexr     ;Yes. Clear index

        movlw   b'00111000'     ;Set input data mask
        andwf   porta,w         ;Read input data
        movwf   input_data      ;Save input data
        rrf     input_data,f    ;Rotate Right
        rrf     input_data,f    ;Rotate Right
        rrf     input_data,f    ;Rotate Right
        movlw   b'00000111'     ;Set pick up mask
        andwf   input_data,f    ;Pick up input data

        movfw   input_data      ;Read input data
        btfss   status,z        ;Input data = 0 ?
        goto    p002            ;No. Next

;*** Sunday
p001
        movlw   sunday          ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p001            ;Next line
        goto    p_end           ;Jump to end

p002
        movlw   d'1'            ;Set check data
        subwf   input_data,w    ;Check input data
        btfss   status,z        ;Input data = 1 ?
        goto    p004            ;Jump to end

;*** Monday
p003
        movlw   monday          ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p003            ;Next line
        goto    p_end           ;Jump to end

p004
        movlw   d'2'            ;Set check data
        subwf   input_data,w    ;Check input data
        btfss   status,z        ;Input data = 2 ?
        goto    p006            ;Jump to end

;*** Tuesday
p005
        movlw   tuesday         ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p005            ;Next line
        goto    p_end           ;Jump to end

p006
        movlw   d'3'            ;Set check data
        subwf   input_data,w    ;Check input data
        btfss   status,z        ;Input data = 3 ?
        goto    p008            ;Jump to end

;*** Wednesday
p007
        movlw   wednesday       ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p007            ;Next line
        goto    p_end           ;Jump to end

p008
        movlw   d'4'            ;Set check data
        subwf   input_data,w    ;Check input data
        btfss   status,z        ;Input data = 4 ?
        goto    p010            ;Jump to end

;*** Thursday
p009
        movlw   thursday        ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p009            ;Next line
        goto    p_end           ;Jump to end

p010
        movlw   d'5'            ;Set check data
        subwf   input_data,w    ;Check input data
        btfss   status,z        ;Input data = 5 ?
        goto    p012            ;Jump to end

;*** Friday
p011
        movlw   friday          ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p011            ;Next line
        goto    p_end           ;Jump to end

p012
        movlw   d'6'            ;Set check data
        subwf   input_data,w    ;Check input data
        btfss   status,z        ;Input data = 6 ?
        goto    p014            ;Jump to end

;*** Saturday
p013
        movlw   saturday        ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p013            ;Next line
        goto    p_end           ;Jump to end

;*** Light_ON
p014
        movlw   light_on        ;Read data address
        call    data_set        ;Data set
        btfsc   status,c        ;Over ?
        goto    p014            ;Next line

p_end
        clrf    line_indexw     ;Clear index
        goto    int_end         ;Jump to end

data_set
        call    data_read       ;Read data
        movwf   line_data       ;Save data
        movlw   disp            ;Read display table HA
        addwf   line_indexw,w   ;Table HA + index
        movwf   fsr             ;Set table address
        movfw   line_data       ;Read data
        movwf   indf            ;Write data
        incf    line_indexw,f   ;Index + 1
        movfw   line_indexw     ;Read index
        sublw   d'7'            ;Check upper limit
        return

data_read
        addwf   line_indexw,w   ;DATA HA + Index
        movwf   pcl             ;Read data

int_end
;************* END of interruption process **************
        retfie

;********************************************************
;          END of Week LED control for RCC
;********************************************************

        end
